# this generator based on rails_admin's install generator.
# https://www.github.com/sferik/rails_admin/master/lib/generators/rails_admin/install_generator.rb

require 'rails/generators'

# http://guides.rubyonrails.org/generators.html
# http://rdoc.info/github/wycats/thor/master/Thor/Actions.html

module Tellimus
  class InstallGenerator < Rails::Generators::Base

    def self.source_paths
      [Tellimus::Engine.root, File.expand_path("../templates", __FILE__)]
    end

    include Rails::Generators::Migration

    argument :subscription_owner_model, :type => :string, :required => false, :desc => "Owner of the subscription"
    desc "Tellimus installation generator"

    # Override the attr_accessor generated by 'argument' so that
    # subscription_owner_model is always returned lowercase.
    def subscription_owner_model
      @subscription_owner_model ? @subscription_owner_model.downcase : "user"
    end


    def install

      unless defined?(Tellimus)
        gem("tellimus")
      end

      require "securerandom"
      template "config/initializers/tellimus.rb"

      # Generate subscription.
      generate("model", "subscription braintree_id:string braintree_customer_id:string plan_id:integer payment_signature:string payment_type:string current_price:float #{subscription_owner_model}_id:integer")
      template "app/models/subscription.rb"

      # Add the plans.
      generate("model", "plan name:string braintree_id:string price:float interval:string features:text highlight:boolean display_order:integer")
      template "app/models/plan.rb"

      # Update the owner relationship.
      inject_into_class "app/models/#{subscription_owner_model}.rb", subscription_owner_model.camelize.constantize,
                        "# Added by Tellimus.\n  has_one :subscription\n\n"

      # Install the pricing table.
      copy_file "app/views/tellimus/subscriptions/_social_proof.html.erb"

      # Add webhooks to the route.

      route <<-RUBY

  # Added by Tellimus.
  mount Tellimus::Engine, at: 'tellimus'
  scope module: 'tellimus' do
    get 'pricing' => 'subscriptions#index', as: 'pricing'
  end

RUBY
    end

  end
end
